@@ -17,9 +17,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Import from the Standard Library
-from os import getenv, listdir
-
 # Import from itools
 from itools.core import get_abspath, get_version
 from itools.gettext import register_domain
@@ -32,39 +29,7 @@ import root
 from webpage import WebPage
 
 
-# The version
-__version__ = get_version()
-
-
-# Register the itools domain
-path = get_abspath('locale')
-register_domain('ikaaro', path)
-
-# Register document types
-register_document_type(WebPage)
-register_document_type(Folder)
-register_document_type(File)
-
-# Import ikaaro sub-packages (NOTE must be imported after so they are
-# register after)
-import calendar
-import forum
-import tracker
-try:
-    import docutils
-except ImportError:
-    print "docutils is not installed, wiki deactivated."
-else:
-    import wiki
-    try:
-        import lpod
-    except ImportError:
-        print "lpod is not installed, wiki ODT export deactivated."
-
-
-###########################################################################
 # Check for required software
-###########################################################################
 for name, import_path, reason in [
         ("poppler", "itools.pdf.pdftotext", "PDF indexation"),
         ("wv2", "itools.office.doctotext", "DOC indexation"),
@@ -74,3 +39,20 @@ for name, import_path, reason in [
     except ImportError:
         print '%s: You need to install "%s" and reinstall itools.' % (reason,
                 name)
+
+
+# The version
+__version__ = get_version()
+
+
+# Import required modules
+import users
+
+# Register the itools domain
+path = get_abspath('locale')
+register_domain('ikaaro', path)
+
+# Register document types
+register_document_type(WebPage)
+register_document_type(Folder)
+register_document_type(File)
