@@ -19,32 +19,37 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.core import guess_all_extensions, merge_dicts
-from itools.datatypes import String
+from itools.core import guess_all_extensions
 from itools.gettext import MSG
-from itools.handlers import File as FileHandler, Image as ImageHandler
+from itools.handlers import File as FileHandler
+from itools.handlers import Image as ImageHandler, SVGFile
 from itools.handlers import TARFile, ZIPFile, GzipFile, Bzip2File
 from itools.odf import SXWFile, SXCFile, SXIFile, ODTFile, ODSFile, ODPFile
 from itools.pdf import PDFFile
-from itools.fs import FileName
-from itools.office import MSPowerPoint as MSPowerPointFile, RTF as RTFFile
+from itools.office import RTF as RTFFile
 from itools.office import MSWord as MSWordFile, MSExcel as MSExcelFile
+from itools.office import MSPowerPoint as MSPowerPointFile
+from itools.office import MSWordX as MSWordXFile, MSExcelX as MSExcelXFile
+from itools.office import MSPowerPointX as MSPowerPointXFile
+from itools.web import get_context
 
 # Import from ikaaro
-from registry import register_resource_class
-from resource_ import DBResource
-from workflow import WorkflowAware
-from file_views import File_NewInstance, File_Download, File_View
+from content import Content
+from database import Database
+from fields import Char_Field, File_Field, Owner_Field
+from file_views import File_NewInstance, File_View
 from file_views import File_Edit, File_ExternalEdit, File_ExternalEdit_View
-from file_views import Image_Thumbnail, Image_View, Video_View, Archive_View
+from file_views import File_Download
+from file_views import Image_View, Video_View, Archive_View
 from file_views import Flash_View
+from resource_views import DBResource_GetImage
 
 
 
 ###########################################################################
 # Base File
 ###########################################################################
-class File(WorkflowAware, DBResource):
+class File(Content):
 
     class_id = 'file'
     class_version = '20090122'
@@ -53,24 +58,13 @@ class File(WorkflowAware, DBResource):
         u'Upload office documents, images, media files, etc.')
     class_icon16 = 'icons/16x16/file.png'
     class_icon48 = 'icons/48x48/file.png'
-    class_views = ['view', 'edit', 'externaledit', 'edit_state', 'backlinks',
-                   'commit_log']
-    class_handler = FileHandler
+    class_views = ['view', 'edit', 'externaledit', 'remove', 'subscribe',
+                   'links', 'backlinks', 'commit_log']
 
-
-    @staticmethod
-    def _make_resource(cls, folder, name, body=None, filename=None,
-                     extension=None, **kw):
-        DBResource._make_resource(cls, folder, name, filename=filename, **kw)
-        # Add the body
-        if body is not None:
-            handler = cls.class_handler(string=body)
-            if extension:
-                extension = extension.lower()
-            else:
-                extension = handler.class_extension
-            name = FileName.encode((name, extension, None))
-            folder.set_handler(name, handler)
+    # Fields
+    data = File_Field(required=True, class_handler=FileHandler)
+    filename = Char_Field
+    owner = Owner_Field
 
 
     def get_all_extensions(self):
@@ -82,7 +76,7 @@ class File(WorkflowAware, DBResource):
         elif format == 'application/x-bzip2':
             extensions = ['bz2', 'tbz2']
         else:
-            cls = self.class_handler
+            cls = self.data.class_handler
             extensions = [ x[1:] for x in guess_all_extensions(format) ]
             if cls.class_extension in extensions:
                 extensions.remove(cls.class_extension)
@@ -90,73 +84,11 @@ class File(WorkflowAware, DBResource):
         return extensions
 
 
-    def get_handler(self):
-        # Already loaded
-        if self._handler is not None:
-            return self._handler
-
-        # Not yet loaded
-        database = self.metadata.database
-        fs = database.fs
-        base = self.metadata.key
-        cls = self.class_handler
-
-        # Check the handler exists
-        extensions = self.get_all_extensions()
-        for extension in extensions:
-            name = FileName.encode((self.name, extension, None))
-            key = fs.resolve(base, name)
-            # Found
-            if database.has_handler(key):
-                self._handler = database.get_handler(key, cls=cls)
-                return self._handler
-
-        # Not found, build a dummy one
-        name = FileName.encode((self.name, cls.class_extension, None))
-        key = fs.resolve(base, name)
-        handler = cls()
-        database.push_phantom(key, handler)
-        self._handler = handler
-        return self._handler
-
-    handler = property(get_handler, None, None, '')
-
-
-    def rename_handlers(self, new_name):
-        folder = self.parent.handler
-        old_name = self.name
-        for extension in self.get_all_extensions():
-            old = FileName.encode((old_name, extension, None))
-            if folder.has_handler(old):
-                return [(old, FileName.encode((new_name, extension, None)))]
-        return [(None, None)]
-
-
-    #######################################################################
-    # Metadata
-    #######################################################################
-    @classmethod
-    def get_metadata_schema(cls):
-        from obsolete.metadata import History
-        return merge_dicts(
-            DBResource.get_metadata_schema(),
-            WorkflowAware.get_metadata_schema(),
-            filename=String,
-            history=History, # XXX Backwards compatibility with 0.50
-            )
-
-
     #######################################################################
     # Versioning & Indexing
     #######################################################################
     def to_text(self):
-        return self.handler.to_text()
-
-
-    def get_size(self):
-        sizes = [ len(x.to_str()) for x in self.get_handlers() ]
-        # XXX Maybe not the good algo
-        return max(sizes)
+        return self.get_value('data').to_text()
 
 
     def get_files_to_archive(self, content=False):
@@ -171,57 +103,73 @@ class File(WorkflowAware, DBResource):
     #######################################################################
     # User Interface
     #######################################################################
-    def get_human_size(self):
-        file = self.handler
-        bytes = len(file.to_str())
-        size = bytes / 1024.0
-        if size >= 1024:
-            size = size / 1024.0
-            str = MSG(u'%.01f MB')
-        else:
-            str = MSG(u'%.01f KB')
-
-        return str.gettext() % size
-
-
     def get_content_type(self):
-        return self.handler.get_mimetype()
+        return self.get_value('data').get_mimetype()
 
     # Views
-    new_instance = File_NewInstance()
-    download = File_Download()
-    view = File_View()
-    edit = File_Edit()
-    externaledit = File_ExternalEdit_View()
-    external_edit = File_ExternalEdit()
+    new_instance = File_NewInstance
+    download = File_Download
+    view = File_View
+    edit = File_Edit
+    externaledit = File_ExternalEdit_View
+    external_edit = File_ExternalEdit
 
 
-    #######################################################################
-    # Update
-    #######################################################################
-    def update_20090122(self):
-        WorkflowAware.update_20090122(self)
-
-        metadata = self.metadata
-        if metadata.has_property('history'):
-            metadata.del_property('history')
-
 
 ###########################################################################
 # Media
 ###########################################################################
 class Image(File):
+
     class_id = 'image'
     class_title = MSG(u'Image')
     class_icon16 = 'icons/16x16/image.png'
     class_icon48 = 'icons/48x48/image.png'
-    class_views = ['view', 'download', 'edit', 'externaledit', 'edit_state',
-                   'backlinks', 'commit_log']
-    class_handler = ImageHandler
+    class_views = ['view', 'download', 'edit', 'externaledit', 'remove',
+                   'links', 'backlinks', 'commit_log']
+    # Fields
+    data = File.data(class_handler=ImageHandler)
+
+    def get_max_width(self):
+        # Auto-reduce width on init
+        server = get_context().server
+        if server is not None:
+            return server.config.get_value('max-width')
+        return None
+
+
+    def get_max_height(self):
+        # Auto-reduce height on init
+        server = get_context().server
+        if server is not None:
+            return server.config.get_value('max-height')
+        return None
+
+
+    def init_resource(self, **kw):
+        super(Image, self).init_resource(**kw)
+        # Resize image at max size
+        max_width = self.get_max_width()
+        max_height = self.get_max_height()
+        if max_width or max_height:
+            handler = self.get_value('data')
+            xsize, ysize = handler.get_size()
+            thumb, format = handler.get_thumbnail(
+                min(xsize, max_width or xsize),
+                min(ysize, max_height or ysize))
+            handler.load_state_from_string(thumb)
 
     # Views
-    thumb = Image_Thumbnail()
-    view = Image_View()
+    thumb = DBResource_GetImage(field_name='data')
+    view = Image_View
+
+
+
+class SVG(Image):
+    class_id = 'image/svg+xml'
+    class_title = MSG(u'Image SVG')
+    # Fields
+    data = Image.data(class_handler=SVGFile)
 
 
 
@@ -233,7 +181,7 @@ class Video(File):
     class_icon48 = 'icons/48x48/flash.png'
 
     # Views
-    view = Video_View()
+    view = Video_View
 
 
 
@@ -245,7 +193,7 @@ class Flash(File):
     class_icon48 = 'icons/48x48/flash.png'
 
     # Views
-    view = Flash_View()
+    view = Flash_View
 
 
 
@@ -258,7 +206,8 @@ class MSWord(File):
     class_description = MSG(u'Word Text')
     class_icon16 = 'icons/16x16/word.png'
     class_icon48 = 'icons/48x48/word.png'
-    class_handler = MSWordFile
+    # Fields
+    data = File.data(class_handler=MSWordFile)
 
 
 
@@ -268,7 +217,8 @@ class MSExcel(File):
     class_description = MSG(u'Excel Spreadsheet')
     class_icon16 = 'icons/16x16/excel.png'
     class_icon48 = 'icons/48x48/excel.png'
-    class_handler = MSExcelFile
+    # Fields
+    data = File.data(class_handler=MSExcelFile)
 
 
 
@@ -278,7 +228,41 @@ class MSPowerPoint(File):
     class_description = MSG(u'PowerPoint Presentation')
     class_icon16 = 'icons/16x16/powerpoint.png'
     class_icon48 = 'icons/48x48/powerpoint.png'
-    class_handler = MSPowerPointFile
+    # Fields
+    data = File.data(class_handler=MSPowerPointFile)
+
+
+
+class MSWordX(File):
+    class_id = MSWordXFile.class_mimetypes[0]
+    class_title = MSG(u'Word')
+    class_description = MSG(u'Word Text')
+    class_icon16 = 'icons/16x16/word.png'
+    class_icon48 = 'icons/48x48/word.png'
+    # Fields
+    data = File.data(class_handler=MSWordXFile)
+
+
+
+class MSExcelX(File):
+    class_id = MSExcelXFile.class_mimetypes[0]
+    class_title = MSG(u'Excel')
+    class_description = MSG(u'Excel Spreadsheet')
+    class_icon16 = 'icons/16x16/excel.png'
+    class_icon48 = 'icons/48x48/excel.png'
+    # Fields
+    data = File.data(class_handler=MSExcelXFile)
+
+
+
+class MSPowerPointX(File):
+    class_id = MSPowerPointXFile.class_mimetypes[0]
+    class_title = MSG(u'PowerPoint')
+    class_description = MSG(u'PowerPoint Presentation')
+    class_icon16 = 'icons/16x16/powerpoint.png'
+    class_icon48 = 'icons/48x48/powerpoint.png'
+    # Fields
+    data = File.data(class_handler=MSPowerPointXFile)
 
 
 
@@ -288,7 +272,8 @@ class OOWriter(File):
     class_description = MSG(u'OpenOffice.org Text')
     class_icon16 = 'icons/16x16/oowriter.png'
     class_icon48 = 'icons/48x48/oowriter.png'
-    class_handler = SXWFile
+    # Fields
+    data = File.data(class_handler=SXWFile)
 
 
 
@@ -298,7 +283,8 @@ class OOCalc(File):
     class_description = MSG(u'OpenOffice.org Spreadsheet')
     class_icon16 = 'icons/16x16/oocalc.png'
     class_icon48 = 'icons/48x48/oocalc.png'
-    class_handler = SXCFile
+    # Fields
+    data = File.data(class_handler=SXCFile)
 
 
 
@@ -308,7 +294,8 @@ class OOImpress(File):
     class_description = MSG(u'OpenOffice.org Presentation')
     class_icon16 = 'icons/16x16/ooimpress.png'
     class_icon48 = 'icons/48x48/ooimpress.png'
-    class_handler = SXIFile
+    # Fields
+    data = File.data(class_handler=SXIFile)
 
 
 
@@ -318,7 +305,8 @@ class PDF(File):
     class_description = MSG(u'PDF Document')
     class_icon16 = 'icons/16x16/pdf.png'
     class_icon48 = 'icons/48x48/pdf.png'
-    class_handler = PDFFile
+    # Fields
+    data = File.data(class_handler=PDFFile)
 
 
 
@@ -328,7 +316,8 @@ class RTF(File):
     class_description = MSG(u'RTF Document')
     class_icon16 = 'icons/16x16/text.png'
     class_icon48 = 'icons/48x48/text.png'
-    class_handler = RTFFile
+    # Fields
+    data = File.data(class_handler=RTFFile)
 
 
 
@@ -338,7 +327,8 @@ class ODT(File):
     class_description = MSG(u'OpenDocument Text')
     class_icon16 = 'icons/16x16/odt.png'
     class_icon48 = 'icons/48x48/odt.png'
-    class_handler = ODTFile
+    # Fields
+    data = File.data(class_handler=ODTFile)
 
 
 
@@ -348,7 +338,8 @@ class ODS(File):
     class_description = MSG(u'OpenDocument Spreadsheet')
     class_icon16 = 'icons/16x16/ods.png'
     class_icon48 = 'icons/48x48/ods.png'
-    class_handler = ODSFile
+    # Fields
+    data = File.data(class_handler=ODSFile)
 
 
 
@@ -358,7 +349,8 @@ class ODP(File):
     class_description = MSG(u'OpenDocument Presentation')
     class_icon16 = 'icons/16x16/odp.png'
     class_icon48 = 'icons/48x48/odp.png'
-    class_handler = ODPFile
+    # Fields
+    data = File.data(class_handler=ODPFile)
 
 
 
@@ -367,7 +359,7 @@ class ODP(File):
 ###########################################################################
 class Archive(File):
 
-    view = Archive_View()
+    view = Archive_View
 
 
 
@@ -377,7 +369,8 @@ class ZipArchive(Archive):
     class_description = MSG(u'Zip Archive')
     class_icon16 = 'icons/16x16/zip.png'
     class_icon48 = 'icons/48x48/zip.png'
-    class_handler = ZIPFile
+    # Fields
+    data = Archive.data(class_handler=ZIPFile)
 
 
 
@@ -387,7 +380,8 @@ class TarArchive(Archive):
     class_description = MSG(u'Tar Archive')
     class_icon16 = 'icons/16x16/tar.png'
     class_icon48 = 'icons/48x48/tar.png'
-    class_handler = TARFile
+    # Fields
+    data = Archive.data(class_handler=TARFile)
 
 
 
@@ -397,7 +391,8 @@ class Gzip(File):
     class_description = MSG(u'Gzip Compressed')
     class_icon16 = 'icons/16x16/gzip.png'
     class_icon48 = 'icons/48x48/gzip.png'
-    class_handler = GzipFile
+    # Fields
+    data = File.data(class_handler=GzipFile)
 
 
 
@@ -407,10 +402,12 @@ class Bzip2(File):
     class_description = MSG(u'Bzip2 Compressed')
     class_icon16 = 'icons/16x16/bzip.png'
     class_icon48 = 'icons/48x48/bzip.png'
-    class_handler = Bzip2File
+    # Fields
+    data = File.data(class_handler=Bzip2File)
 
 
 ###########################################################################
 # Register
 ###########################################################################
-register_resource_class(File, format="application/octet-stream")
+Database.register_resource_class(File, 'application/octet-stream')
+Database.register_resource_class(ZipArchive, 'application/x-zip-compressed')
