@@ -14,9 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Import from the Standard Library
-from urllib import quote
-
 # Import from the Python Image Library
 try:
     from PIL import Image as PILImage
@@ -25,25 +22,65 @@ except ImportError:
 
 # Import from itools
 from itools.core import merge_dicts
-from itools.datatypes import Boolean, Integer, String, Unicode
+from itools.database import AndQuery, NotQuery, PhraseQuery, OrQuery, TextQuery
+from itools.datatypes import Boolean, Enumerate, Integer, String, Unicode
 from itools.gettext import MSG
 from itools.handlers import checkid
-from itools.i18n import format_datetime
-from itools.stl import set_prefix
+from itools.handlers.utils import transmap
+from itools.html import stream_is_empty
 from itools.uri import get_reference, Path
-from itools.web import BaseView, STLForm, ERROR
-from itools.xapian import AndQuery, OrQuery, PhraseQuery, split_unicode
+from itools.web import BaseView, STLView, get_context
 
 # Import from ikaaro
-from buttons import RemoveButton, RenameButton, CopyButton, CutButton
-from buttons import PasteButton, PublishButton, RetireButton
-from datatypes import CopyCookie, ImageWidth
+from autoform import SelectWidget, TextWidget
+from buttons import PasteButton
+from buttons import Remove_BrowseButton, RenameButton, CopyButton, CutButton
+from buttons import ZipButton
+from datatypes import CopyCookie
 from exceptions import ConsistencyError
+from utils import generate_name, get_base_path_query, get_content_containers
+from views import IconsView, BrowseForm, ContextMenu
 import messages
-from utils import generate_name, get_base_path_query
-from views import IconsView, SearchForm, ContextMenu
-from views_new import AddResourceMenu
-from workflow import WorkflowAware, get_workflow_preview
+
+
+class SearchTypes_Enumerate(Enumerate):
+
+    def get_options(self):
+        context = get_context()
+        resource = context.resource
+        view = context.view
+        # 1. Build the query of all objects to search
+        query = get_base_path_query(resource.abspath)
+        if view.search_content_only(resource, context) is True:
+            content_query = PhraseQuery('is_content', True)
+            query = AndQuery(query, content_query)
+
+        # 2. Compute children_formats
+        children_formats = set()
+        for child in context.search(query).get_documents():
+            children_formats.add(child.format)
+
+        # 3. Do not show two options with the same title
+        formats = {}
+        for type in children_formats:
+            cls = context.database.get_resource_class(type)
+            title = cls.class_title.gettext()
+            formats.setdefault(title, []).append(type)
+
+        # 4. Build the namespace
+        types = []
+        for title, type in formats.items():
+            type = ','.join(type)
+            types.append({'name': type, 'value': title})
+        types.sort(key=lambda x: x['value'].lower())
+
+        return types
+
+
+    # This is done to fix an error: select one resource type, remove all
+    # resources, then traceback.
+    def is_valid(self, value):
+        return True
 
 
 
@@ -51,17 +88,18 @@ class ZoomMenu(ContextMenu):
 
     title = MSG(u'Zoom')
 
-    def get_items(self, resource, context):
-        uri = context.uri
+    def get_items(self):
+        uri = self.context.uri
 
         # Compute previous and next sizes
-        current_size = context.query['size']
-        min_size = resource.MIN_SIZE
-        max_size = resource.MAX_SIZE
+        current_size = self.context.query['size']
+        size_steps = self.resource.SIZE_STEPS
+        min_size = size_steps[0]
+        max_size = size_steps[-1]
         current_size = max(min_size, min(current_size, max_size))
         previous_size = min_size
         next_size = max_size
-        for step in resource.SIZE_STEPS:
+        for step in size_steps:
             if step < current_size:
                 previous_size = step
             if next_size is max_size and step > current_size:
@@ -87,18 +125,9 @@ class Folder_View(BaseView):
 
 
     def GET(self, resource, context):
-        from webpage import WebPage
-
-        index = resource.get_resource('index', soft=True)
-        if not isinstance(index, WebPage):
-            context.message = ERROR(
-                u'There is not an "index" web page. Could not render this '
-                u'view.')
-            return ''
-
-        # Rewrite the URLs
-        stream = index.get_html_data()
-        return set_prefix(stream, 'index/')
+        stream = resource.get_html_field_body_stream('index')
+        stream = list(stream)
+        return stream if not stream_is_empty(stream) else ''
 
 
 
@@ -109,22 +138,57 @@ class Folder_NewResource(IconsView):
     icon = 'new.png'
 
 
+    def GET(self, resource, context):
+        # If only one type of event, we redirect on it
+        items = self.get_items(resource, context)
+        if len(items) == 1:
+            return self.get_url(items[0].class_id, context)
+
+        return super(Folder_NewResource, self).GET(resource, context)
+
+
+    def get_url(self, class_id, context):
+        query = context.uri.query.copy()
+        query['type'] = class_id
+        query['referrer'] = context.get_referrer()
+        uri = context.uri.resolve('./;new_resource')
+        uri.query = query
+        return uri
+
+
+    def get_items(self, resource, context):
+        # 1. Static classes
+        aux = set()
+        document_types = []
+        for container in get_content_containers(context):
+            if container.class_id in aux:
+                continue
+            aux.add(container.class_id)
+            for cls in container.get_document_types():
+                if cls not in document_types:
+                    document_types.append(cls)
+
+        # 2. Add dynamic models
+        for cls in context.database.get_dynamic_classes():
+            document_types.append(cls)
+
+        # Ok
+        return document_types
+
+
     def get_namespace(self, resource, context):
         items = [
             {'icon': '/ui/' + cls.class_icon48,
-             'title': cls.class_title.gettext(),
-             'description': cls.class_description.gettext(),
-             'url': ';new_resource?type=%s' % quote(cls.class_id)
-            }
-            for cls in resource.get_document_types() ]
+             'title': cls.class_title,
+             'description': cls.class_description,
+             'url': self.get_url(cls.class_id, context)}
+            for cls in self.get_items(resource, context) ]
 
-        return {
-            'batch': None,
-            'items': items}
+        return {'batch': None, 'items': items}
 
 
 
-class Folder_Rename(STLForm):
+class Folder_Rename(STLView):
 
     access = 'is_allowed_to_edit'
     title = MSG(u'Rename resources')
@@ -140,10 +204,10 @@ class Folder_Rename(STLForm):
     def get_namespace(self, resource, context):
         ids = context.query['ids']
         # Filter names which the authenticated user is not allowed to move
-        ac = resource.get_access_control()
+        root = context.root
         user = context.user
         paths = [ x for x in ids
-                  if ac.is_allowed_to_move(user, resource.get_resource(x)) ]
+                  if root.is_allowed_to_move(user, resource.get_resource(x)) ]
 
         # Build the namespace
         paths.sort()
@@ -176,7 +240,7 @@ class Folder_Rename(STLForm):
         renamed = []
         referenced = []
         # Process input data
-        abspath = resource.get_abspath()
+        abspath = resource.abspath
         for i, path in enumerate(paths):
             new_name = new_names[i]
             new_name = checkid(new_name)
@@ -219,71 +283,178 @@ class Folder_Rename(STLForm):
 
 
 
-class Folder_BrowseContent(SearchForm):
+class Folder_BrowseContent(BrowseForm):
 
     access = 'is_allowed_to_view'
     title = MSG(u'Browse Content')
-    context_menus = [AddResourceMenu()]
-    query_schema = merge_dicts(SearchForm.query_schema,
+    context_menus = []
+    query_schema = merge_dicts(BrowseForm.query_schema,
         sort_by=String(default='mtime'),
         reverse=Boolean(default=True))
     schema = {
-        'ids': String(multiple=True, mandatory=True),
-    }
+        'ids': String(multiple=True, mandatory=True)}
 
     # Search Form
-    search_template = '/ui/folder/browse_search.xml'
+    search_widgets = [
+        TextWidget('text', title=MSG(u'Text')),
+        SelectWidget('format', title=MSG(u'Type'))]
     search_schema = {
-        'search_field': String,
-        'search_term': Unicode,
-        'search_subfolders': Boolean(default=False),
-    }
+        'text': Unicode,
+        'format': SearchTypes_Enumerate}
 
     # Table
     table_columns = [
         ('checkbox', None),
         ('icon', None),
-        ('name', MSG(u'Name')),
+        ('abspath', MSG(u'Path')),
         ('title', MSG(u'Title')),
         ('format', MSG(u'Type')),
         ('mtime', MSG(u'Last Modified')),
-        ('last_author', MSG(u'Last Author')),
-        ('size', MSG(u'Size')),
-        ('workflow_state', MSG(u'State'))]
+        ('last_author', MSG(u'Last Author'))]
+    table_actions = [
+        Remove_BrowseButton, RenameButton,
+        CopyButton, CutButton, PasteButton,
+        ZipButton]
 
 
-    def get_search_namespace(self, resource, context):
-        namespace = SearchForm.get_search_namespace(self, resource, context)
-        namespace['search_subfolders'] = context.query['search_subfolders']
-        return namespace
+    def search_content_only(self, resource, context):
+        return resource.is_content
 
 
-    def get_items(self, resource, context, *args):
-        # Get the parameters from the query
-        query = context.query
-        search_term = query['search_term'].strip()
-        field = query['search_field']
-        search_subfolders = query['search_subfolders']
+    def get_scripts(self, context):
+        scripts = []
+        if self.search_widgets:
+            for widget in self.search_widgets:
+                for script in widget.scripts:
+                    if script not in scripts:
+                        scripts.append(script)
 
-        # Build the query
-        args = list(args)
-        abspath = str(resource.get_canonical_path())
-        if search_subfolders is True:
-            args.append(get_base_path_query(abspath))
-        else:
-            args.append(PhraseQuery('parent_path', abspath))
-        if search_term:
-            language = resource.get_content_language(context)
-            terms_query = [ PhraseQuery(field, term)
-                            for term in split_unicode(search_term, language) ]
-            args.append(AndQuery(*terms_query))
-        if len(args) == 1:
-            query = args[0]
-        else:
-            query = AndQuery(*args)
+        return scripts
 
-        # Ok
-        return context.root.search(query)
+
+    def get_styles(self, context):
+        styles = []
+        if self.search_widgets:
+            for widget in self.search_widgets:
+                for style in widget.styles:
+                    if style not in styles:
+                        styles.append(style)
+        return styles
+
+
+    depth = None
+    base_classes = None
+    def get_items_query(self, resource, context):
+        # Search in subtree
+        query = get_base_path_query(resource.abspath, max_depth=self.depth)
+
+        # Base classes
+        base_classes = self.base_classes
+        if base_classes is not None:
+            base_classes_query = OrQuery(*
+                [ PhraseQuery('base_classes', x) for x in base_classes ])
+            query = AndQuery(query, base_classes_query)
+
+        # Exclude non-content
+        if self.search_content_only(resource, context) is True:
+            query = AndQuery(query, PhraseQuery('is_content', True))
+
+        return query
+
+
+    def get_search_query(self, resource, context):
+        query = AndQuery()
+        form = context.query
+        for key, datatype in self.search_schema.items():
+            value = form[key]
+            if value is None or value == '':
+                continue
+            # Special case: search on text, title and name AS AndQuery
+            if key == 'text':
+                text_query = []
+                value = value.split(' ')
+                for v in value:
+                    t_query = OrQuery(TextQuery('title', v),
+                                      TextQuery('text', v),
+                                      PhraseQuery('name', v))
+                    text_query.append(t_query)
+                if len(text_query) == 1:
+                    text_query = text_query[0]
+                else:
+                    text_query = AndQuery(*text_query)
+                query.append(text_query)
+            # Special case: type
+            elif key == 'format':
+                squery = [ PhraseQuery('format', x) for x in value.split(',') ]
+                squery = squery[0] if len(squery) == 1 else OrQuery(*squery)
+                query.append(squery)
+            # Multiple
+            elif datatype.multiple is True:
+                query.append(OrQuery(*[ PhraseQuery(key, x) for x in value ]))
+            # Singleton
+            else:
+                if value is False:
+                    # FIXME No value means False in xapian
+                    query.append(NotQuery(PhraseQuery(key, True)))
+                else:
+                    query.append(PhraseQuery(key, value))
+        return query
+
+
+    def get_items(self, resource, context):
+        # Base query
+        query = self.get_items_query(resource, context)
+
+        # Search form
+        search_query = self.get_search_query(resource, context)
+        if search_query:
+            query = AndQuery(query, search_query)
+
+        # Search
+        return context.search(query)
+
+
+    def _get_key_sorted_by_unicode(self, field):
+        def key(item):
+            return getattr(item, field).lower().translate(transmap)
+        return key
+
+
+    def _get_key_sorted_by_user(self, field):
+        get_user_title = self.context.root.get_user_title
+        def key(item, cache={}):
+            user = getattr(item, field)
+            if user in cache:
+                return cache[user]
+            if user:
+                title = get_user_title(user)
+                value = title.lower().translate(transmap)
+            else:
+                value = None
+            cache[user] = value
+            return value
+        return key
+
+
+    def get_key_sorted_by_title(self):
+        return self._get_key_sorted_by_unicode('title')
+
+
+    def get_key_sorted_by_format(self):
+        database = self.context.database
+        def key(item, cache={}):
+            format = item.format
+            if format in cache:
+                return cache[format]
+            cls = database.get_resource_class(format)
+            value = cls.class_title.gettext().lower().translate(transmap)
+            cache[format] = value
+            return value
+        return key
+
+
+    def get_key_sorted_by_last_author(self):
+        return self._get_key_sorted_by_user('last_author')
 
 
     def sort_and_batch(self, resource, context, results):
@@ -291,79 +462,74 @@ class Folder_BrowseContent(SearchForm):
         size = context.query['batch_size']
         sort_by = context.query['sort_by']
         reverse = context.query['reverse']
-        items = results.get_documents(sort_by=sort_by, reverse=reverse,
-                                      start=start, size=size)
 
-        # FIXME This must be done in the catalog.
-        if sort_by == 'title':
-            items.sort(cmp=lambda x,y: cmp(x.title, y.title))
-            if reverse:
-                items.reverse()
+        if sort_by is None:
+            get_key = None
+        else:
+            get_key = getattr(self, 'get_key_sorted_by_' + sort_by, None)
 
-        # Access Control (FIXME this should be done before batch)
-        user = context.user
-        root = context.root
-        allowed_items = []
-        for item in items:
-            resource = root.get_resource(item.abspath)
-            ac = resource.get_access_control()
-            if ac.is_allowed_to_view(user, resource):
-                allowed_items.append((item, resource))
+        # Case 1: Custom but slower sort algorithm
+        if get_key:
+            items = results.get_documents()
+            items.sort(key=get_key(), reverse=reverse)
+            if size:
+                items = items[start:start+size]
+            elif start:
+                items = items[start:]
+            database = resource.database
+            return [ database.get_resource(x.abspath) for x in items ]
 
-        return allowed_items
+        # Case 2: Faster Xapian sort algorithm
+        items = results.get_resources(sort_by, reverse, start, size)
+        return list(items)
 
 
     def get_item_value(self, resource, context, item, column):
-        brain, item_resource = item
         if column == 'checkbox':
             # checkbox
-            parent = item_resource.parent
+            parent = item.parent
             if parent is None:
                 return None
-            if item_resource.name in parent.__fixed_handlers__:
+            if item.name in parent.__fixed_handlers__:
                 return None
-            id = resource.get_canonical_path().get_pathto(brain.abspath)
+            id = resource.abspath.get_pathto(item.abspath)
             id = str(id)
             return id, False
         elif column == 'icon':
             # icon
-            path_to_icon = item_resource.get_resource_icon(16)
+            path_to_icon = item.get_resource_icon(16)
+            if not path_to_icon:
+                return None
             if path_to_icon.startswith(';'):
-                path_to_icon = Path('%s/' % brain.name).resolve(path_to_icon)
+                path_to_icon = Path('%s/' % item.name).resolve(path_to_icon)
             return path_to_icon
-        elif column == 'name':
+        elif column == 'abspath':
             # Name
-            id = resource.get_canonical_path().get_pathto(brain.abspath)
+            id = resource.abspath.get_pathto(item.abspath)
             id = str(id)
-            view = item_resource.get_view(None)
+            view = item.get_view(None)
             if view is None:
                 return id
-            href = '%s/' % context.get_link(item_resource)
+            href = '%s/' % context.get_link(item)
             return id, href
-        elif column == 'title':
-            # Title
-            return item_resource.get_property('title')
         elif column == 'format':
             # Type
-            return item_resource.class_title.gettext()
+            return item.class_title.gettext()
         elif column == 'mtime':
             # Last Modified
-            accept = context.accept_language
-            return format_datetime(brain.mtime, accept=accept)
+            mtime = item.get_value('mtime')
+            if mtime:
+                return context.format_datetime(mtime)
+            return None
         elif column == 'last_author':
             # Last author
-            return context.root.get_user_title(brain.last_author)
-        elif column == 'size':
-            # Size
-            return item_resource.get_human_size()
-        elif column == 'workflow_state':
-            # The workflow state
-            return get_workflow_preview(item_resource, context)
+            author =  item.get_value('last_author')
+            return context.root.get_user_title(author) if author else None
+        elif column == 'row_css':
+            return None
 
-
-    table_actions = [
-        RemoveButton, RenameButton, CopyButton, CutButton, PasteButton,
-        PublishButton, RetireButton]
+        # Default
+        return item.get_value_title(column)
 
 
     #######################################################################
@@ -372,15 +538,12 @@ class Folder_BrowseContent(SearchForm):
     def action_remove(self, resource, context, form):
         ids = form['ids']
 
-        # Clean the copy cookie if needed
-        cut, paths = context.get_cookie('ikaaro_cp', datatype=CopyCookie)
-
         # Remove resources
         removed = []
         referenced = []
         not_removed = []
         user = context.user
-        abspath = resource.get_abspath()
+        root = context.root
 
         # We sort and reverse ids in order to
         # remove the childs then their parents
@@ -388,8 +551,7 @@ class Folder_BrowseContent(SearchForm):
         ids.reverse()
         for name in ids:
             child = resource.get_resource(name)
-            ac = child.get_access_control()
-            if ac.is_allowed_to_remove(user, child):
+            if root.is_allowed_to_remove(user, child):
                 # Remove resource
                 try:
                     resource.del_resource(name)
@@ -397,10 +559,6 @@ class Folder_BrowseContent(SearchForm):
                     referenced.append(name)
                     continue
                 removed.append(name)
-                # Clean cookie
-                if str(abspath.resolve2(name)) in paths:
-                    context.del_cookie('ikaaro_cp')
-                    paths = []
             else:
                 not_removed.append(name)
 
@@ -410,8 +568,14 @@ class Folder_BrowseContent(SearchForm):
             msg = messages.MSG_RESOURCES_REMOVED(resources=resources)
             message.append(msg)
         if referenced:
-            resources = ', '.join(referenced)
-            msg = messages.MSG_RESOURCES_REFERENCED(resources=resources)
+            items = []
+            for name in referenced:
+                item = resource.get_resource(name)
+                view = item.get_view('backlinks')
+                if context.is_access_allowed(item, view):
+                    items.append({'title': item.get_title(),
+                                  'href': '%s/;backlinks' % item.abspath})
+            msg = messages.MSG_RESOURCES_REFERENCED_HTML(resources=items)
             message.append(msg)
         if not_removed:
             resources = ', '.join(not_removed)
@@ -425,10 +589,10 @@ class Folder_BrowseContent(SearchForm):
     def action_rename(self, resource, context, form):
         ids = form['ids']
         # Filter names which the authenticated user is not allowed to move
-        ac = resource.get_access_control()
+        root = context.root
         user = context.user
         paths = [ x for x in ids
-                  if ac.is_allowed_to_move(user, resource.get_resource(x)) ]
+                  if root.is_allowed_to_move(user, resource.get_resource(x)) ]
 
         # Check input data
         if not paths:
@@ -447,17 +611,17 @@ class Folder_BrowseContent(SearchForm):
     def action_copy(self, resource, context, form):
         ids = form['ids']
         # Filter names which the authenticated user is not allowed to copy
-        ac = resource.get_access_control()
+        root = context.root
         user = context.user
         names = [ x for x in ids
-                  if ac.is_allowed_to_copy(user, resource.get_resource(x)) ]
+                  if root.is_allowed_to_copy(user, resource.get_resource(x)) ]
 
         # Check input data
         if not names:
-            message = messages.MSG_NONE_SELECTED
+            context.message = messages.MSG_NONE_SELECTED
             return
 
-        abspath = resource.get_abspath()
+        abspath = resource.abspath
         cp = (False, [ str(abspath.resolve2(x)) for x in names ])
         cp = CopyCookie.encode(cp)
         context.set_cookie('ikaaro_cp', cp, path='/')
@@ -468,17 +632,17 @@ class Folder_BrowseContent(SearchForm):
     def action_cut(self, resource, context, form):
         ids = form['ids']
         # Filter names which the authenticated user is not allowed to move
-        ac = resource.get_access_control()
+        root = context.root
         user = context.user
         names = [ x for x in ids
-                  if ac.is_allowed_to_move(user, resource.get_resource(x)) ]
+                  if root.is_allowed_to_move(user, resource.get_resource(x)) ]
 
         # Check input data
         if not names:
             context.message = messages.MSG_NONE_SELECTED
             return
 
-        abspath = resource.get_abspath()
+        abspath = resource.abspath
         cp = (True, [ str(abspath.resolve2(x)) for x in names ])
         cp = CopyCookie.encode(cp)
         context.set_cookie('ikaaro_cp', cp, path='/')
@@ -529,12 +693,6 @@ class Folder_BrowseContent(SearchForm):
                     continue
                 else:
                     pasted.append(name)
-                    # Fix state
-                    copy = target.get_resource(name)
-                    if isinstance(copy, WorkflowAware):
-                        metadata = copy.metadata
-                        metadata.set_property('state',
-                                              copy.workflow.initstate)
 
         # Cut, clean cookie
         if cut is True:
@@ -552,37 +710,15 @@ class Folder_BrowseContent(SearchForm):
         context.message = message
 
 
-    def _action_workflow(self, resource, context, form, transition, statename,
-                         message):
-        resources = [ resource.get_resource(id) for id in form['ids'] ]
-        user = context.user
-        # Check there is at least one item we can publish
-        ac = resource.get_access_control()
-        allowed = [ x for x in resources
-                    if ac.is_allowed_to_trans(user, x, transition) ]
-        if not allowed:
-            context.message = messages.MSG_NONE_ALLOWED
-            return
-
-        # Publish
-        for item in allowed:
-            if item.get_statename() == statename:
-                continue
-            # Update workflow history
-            item.make_transition(transition)
-
+    def action_zip(self, resource, context, form):
+        names = sorted(form['ids'], reverse=True)
+        data = resource.export_zip(names)
+        # Content-Type & Content-Disposition
+        context.set_content_type('application/zip')
+        filename = 'archive.zip'
+        context.set_content_disposition('inline', filename)
         # Ok
-        context.message = message
-
-
-    def action_publish(self, resource, context, form):
-        self._action_workflow(resource, context, form, 'publish', 'public',
-                              messages.MSG_PUBLISHED)
-
-
-    def action_retire(self, resource, context, form):
-        self._action_workflow(resource, context, form, 'retire', 'private',
-                              messages.MSG_RETIRED)
+        return data
 
 
 
@@ -590,30 +726,29 @@ class Folder_PreviewContent(Folder_BrowseContent):
 
     title = MSG(u'Preview Content')
     styles = ['/ui/gallery/style.css']
-    scripts = ['/ui/gallery/javascript.js']
 
     context_menus = Folder_BrowseContent.context_menus + [ZoomMenu()]
-    # Table
-    table_template = '/ui/folder/browse_image.xml'
+
+
+    base_classes = ('image',)
 
 
     def get_query_schema(self):
         # Define a huge batch limit, and the image size parameter
-        return merge_dicts(Folder_BrowseContent.get_query_schema(self),
+        schema = super(Folder_PreviewContent, self).get_query_schema()
+        return merge_dicts(schema,
                            batch_size=Integer(default=0),
                            size=Integer(default=128),
                            width=String,
                            height=String)
 
 
-    def get_items(self, resource, context):
-        # Show only images
-        query = OrQuery(PhraseQuery('is_image', True),
-                        PhraseQuery('format', 'folder'))
-        return Folder_BrowseContent.get_items(self, resource, context, query)
+    # Table
+    table_template = '/ui/folder/browse_image.xml'
 
+    def get_table_head(self, resource, context, items):
+        actions = self.actions_namespace
 
-    def get_table_head(self, resource, context, items, actions=None):
         # Get from the query
         query = context.query
         sort_by = query['sort_by']
@@ -621,7 +756,7 @@ class Folder_PreviewContent(Folder_BrowseContent):
 
         columns = self._get_table_columns(resource, context)
         columns_ns = []
-        for name, title, sortable in columns:
+        for name, title, sortable, css in columns:
             if name == 'checkbox':
                 # Type: checkbox
                 if  actions:
@@ -661,44 +796,27 @@ class Folder_PreviewContent(Folder_BrowseContent):
 
         # (0) Zoom
         current_size = query['size']
-        min_size = resource.MIN_SIZE
-        max_size = resource.MAX_SIZE
+        min_size = resource.SIZE_STEPS[0]
+        max_size = resource.SIZE_STEPS[-1]
         current_size = max(min_size, min(current_size, max_size))
 
         # (1) Actions (submit buttons)
-        ac = resource.get_access_control()
-        actions = []
-        for button in self.get_table_actions(resource, context):
-            if button.show(resource, context, items) is False:
-                continue
-            if button.confirm:
-                confirm = button.confirm.gettext().encode('utf_8')
-                onclick = 'return confirm("%s");' % confirm
-            else:
-                onclick = None
-            actions.append(
-                {'value': button.name,
-                 'title': button.title,
-                 'class': button.css,
-                 'onclick': onclick})
+        self._items = items
+        actions = self.actions_namespace
 
         # (2) Table Head: columns
-        table_head = self.get_table_head(resource, context, items, actions)
+        table_head = self.get_table_head(resource, context, items)
 
         # (3) Table Body: rows
         columns = self._get_table_columns(resource, context)
         rows = []
         for item in items:
-            item_brain, item_resource = item
             row = {'checkbox': False,
                    # These are required for internal use
-                   'title_or_name': item_resource.get_title(),
-                   'workflow_statename': None}
+                   'title_or_name': item.get_title()}
             # XXX Already hard-coded in the catalog search
-            row['is_folder'] = (item_brain.format == 'folder')
-            if isinstance(item_resource, WorkflowAware):
-                row['workflow_statename'] = item_resource.get_statename()
-            for name, title, sortable in columns:
+            row['is_folder'] = (item.class_id == 'folder')
+            for name, title, sortable, css in columns:
                 value = self.get_item_value(resource, context, item, name)
                 if value is None:
                     continue
@@ -708,32 +826,21 @@ class Folder_PreviewContent(Folder_BrowseContent):
                         row['checkbox'] = True
                         row['id'] = value
                         row['checked'] = checked
-                elif name == 'name':
+                elif name == 'abspath':
                     if type(value) is tuple:
-                        value, href = value
-                        href = get_reference(href)
-                        if row['is_folder']:
-                            href = href.resolve_name(';preview_content')
-                        href = href.replace(size=current_size, width=width,
-                                height=height)
-                        href = str(href)
+                        row['name'], row['href'] = value
                     else:
-                        href = None
-                    row['name'] = value
-                    row['href'] = href
+                        row['name'] = value
+                        row['href'] = None
                 else:
                     row[name] = value
             rows.append(row)
 
-        widths = ", ".join([repr(o['name']) for o in ImageWidth.get_options()
-                            if o['name'].strip()])
-
         return {
             'root': resource.parent is None,
             'size': current_size,
             'width': width,
             'height': height,
-            'widths': widths,
             'css': self.table_css,
             'columns': table_head,
             'rows': rows,
@@ -741,47 +848,6 @@ class Folder_PreviewContent(Folder_BrowseContent):
 
 
 
-
-class Folder_Orphans(Folder_BrowseContent):
-    """Orphans are files not referenced in another resource of the database.
-    It extends the concept of "orphans pages" from the wiki to all file-like
-    resources.
-
-    Orphans folders generally don't make sense because they serve as
-    containers. TODO or list empty folders?
-    """
-
-    access = 'is_allowed_to_view'
-    title = MSG(u"Orphans")
-    icon = 'orphans.png'
-    description = MSG(u"Show resources not linked from anywhere.")
-
-
-    def get_items(self, resource, context):
-        # Make the base search
-        items = Folder_BrowseContent.get_items(self, resource, context)
-
-        # Find out the orphans
-        root = context.root
-        orphans = []
-        for item in items.get_documents():
-            query = PhraseQuery('links', item.abspath)
-            results = root.search(query)
-            if len(results) == 0:
-                orphans.append(item)
-
-        # Transform back the items found in a SearchResults object.
-        # FIXME This is required by 'get_item_value', we should change that,
-        # for better performance.
-        args = [ PhraseQuery('abspath', x.abspath) for x in orphans ]
-        query = OrQuery(*args)
-        items = root.search(query)
-
-        # Ok
-        return items
-
-
-
 class Folder_Thumbnail(BaseView):
 
     access = True
@@ -789,32 +855,16 @@ class Folder_Thumbnail(BaseView):
     default_icon = '/ui/gallery/folder.png'
 
     def GET(self, resource, context):
-        from file import Image
-
-        width = context.get_form_value('width', type=Integer, default=48)
-        height = context.get_form_value('height', type=Integer, default=48)
-        size = (width, height)
-        data = None
-        format = "jpeg"
-
-        # Choose an image to illustrate
         default_icon = resource.get_resource(self.default_icon)
         if PILImage is None:
             # Full size but better than nothing
             data = default_icon.to_str()
             format = 'png'
         else:
-            # Find the first accessible image
-            user = context.user
-            ac = resource.get_access_control()
-            for image in resource.search_resources(cls=Image):
-                # Search public image safe for all
-                if ac.is_allowed_to_view(user, image):
-                    data, format = image.handler.get_thumbnail(width, height)
-                    break
-            else:
-                # Default icon for empty or inaccessible folders
-                data, format = default_icon.get_thumbnail(width, height)
+            # Default icon for empty or inaccessible folders
+            width = context.get_form_value('width', type=Integer, default=48)
+            height = context.get_form_value('height', type=Integer, default=48)
+            data, format = default_icon.get_thumbnail(width, height)
 
         # XXX Don't cache nothing here
         # The image thumbnail was cached in the image handler
@@ -857,4 +907,3 @@ class GoToSpecificDocument(BaseView):
             goto = goto.replace(message=message)
 
         return goto
-
