@@ -36,8 +36,6 @@ MSG_CHANGES_SAVED2 = INFO(u'The changes have been saved ({time}).')
 
 MSG_DELETE_RESOURCE = MSG(u'Are you sure you want to delete this resource?')
 
-MSG_DELETE_SELECTION = MSG(u'Are you sure you want to delete the selection?')
-
 MSG_EDIT_CONFLICT = ERROR(
     u'Someone already saved this document, click "Save" again to force.')
 
@@ -65,6 +63,14 @@ MSG_RESOURCES_REMOVED = INFO(u'Resources removed: {resources}.')
 MSG_RESOURCES_REFERENCED = ERROR(
     u'Action impossible (the resources are in use): {resources}.')
 
+MSG_RESOURCES_REFERENCED_HTML = ERROR(u"""
+    Action impossible (the resources are in use):
+    <stl:inline stl:repeat="resource resources">
+        <a href="${resource/href}"
+           title="${resource/title}">${resource/title}</a>
+        <stl:inline stl:if="not repeat/resource/end">,</stl:inline>
+    </stl:inline>""", format='stl')
+
 MSG_RESOURCES_NOT_PASTED = ERROR(u'Resources not allowed to paste here: '
                                  u'{resources}.')
 
@@ -77,6 +83,8 @@ MSG_REGISTERED = ERROR(
     u"You have already confirmed your registration. "
     u"Try to log in or ask for a new password.")
 
+MSG_PASSWORD_EQUAL_TO_USERNAME = ERROR(u'Password cannot match the username.')
+
 MSG_NONE_SELECTED = ERROR(u'No resource selected.')
 
 MSG_NONE_ALLOWED = ERROR(u"No resource allowed.")
@@ -94,3 +102,6 @@ MSG_PUBLISHED = INFO(u'Resources published.')
 MSG_RETIRED = INFO(u'Resources retired.')
 
 MSG_UNEXPECTED_MIMETYPE = ERROR(u'Unexpected file of mimetype {mimetype}.')
+
+MSG_LOGIN_WRONG_NAME_OR_PASSWORD = ERROR(
+    u'The login name or the password is incorrect.')
