@@ -24,14 +24,13 @@ import sys
 
 # Import from itools
 import itools
-from itools.core import start_subprocess
+from itools.database import Metadata
 
 # Import from ikaaro
 from ikaaro.database import make_database
-from ikaaro.resource_ import DBResource
 from ikaaro.root import Root
-from ikaaro.utils import generate_password
 from ikaaro.server import get_fake_context
+from ikaaro.utils import generate_password
 
 
 template = (
@@ -67,37 +66,49 @@ smtp-password =
 # higher verbosity): 'critical' 'error', 'warning', 'info' and 'debug'.
 # The default is 'warning'.
 #
+# If the "log-email" address is defined error messages will be sent to it.
+#
 log-level = warning
+log-email = {log_email}
+
+# The "cron-interval" variable defines the number of seconds between every
+# call to the cron job manager. If zero (the default) the cron job won't be
+# run at all.
+#
+cron-interval = 0
+
+# If the "session-timeout" variable is different from zero (the default), the
+# user will be automatically logged out after the specified number of minutes.
+#
+session-timeout = 0
 
 # The "database-size" variable defines the number of file handlers to store
 # in the database cache.  It is made of two numbers, the upper limit and the
 # bottom limit: when the cache size hits the upper limit, handlers will be
 # removed from the cache until it hits the bottom limit.
 #
-database-size = 4800:5200
-
-# If the "profile-time" variable is set to "1", profiling information will
-# be written to the 'log/profile' file.  If the "profile-space" variable is
-# set to "1", remote monitoring will be enabled (see http://guppy-pe.sf.net/
-# to learn more).
+# The "database-readonly" variable, when set to 1 starts the database in
+# read-only mode, all write operations will fail.
 #
-profile-time = 0
-profile-space = 0
+database-size = 19500:20500
+database-readonly = 0
 
-# The "index-text" vairable defines whether the catalog must process full-text
+# The "index-text" variable defines whether the catalog must process full-text
 # indexing. It requires (much) more time and third-party applications.
 # To speed up catalog updates, set this option to 0 (default is 1).
+#
 index-text = 1
+
+# The size of images can be controlled by setting the following values.
+# (ie. max-width = 1280) (by default it is None, keeping original size).
+#
+max-width =
+max-height =
 """)
 
 
 
 def init(parser, options, target):
-    try:
-        mkdir(target)
-    except OSError:
-        parser.error('can not create the instance (check permissions)')
-
     # Get the email address for the init user
     if options.email is None:
         sys.stdout.write("Type your email address: ")
@@ -114,43 +125,54 @@ def init(parser, options, target):
     # Load the root class
     if options.root is None:
         root_class = Root
-        modules = ''
+        modules = []
     else:
-        modules = options.root
-        exec('import %s' % modules)
-        exec('root_class = %s.Root' % modules)
+        modules = [options.root]
+        exec('import %s' % options.root)
+        exec('root_class = %s.Root' % options.root)
+
+    # Load the modules
+    for module in options.modules.split():
+        modules.append(module)
+        exec('import %s' % module)
+
+    # Make folder
+    try:
+        mkdir(target)
+    except OSError:
+        parser.error('can not create the instance (check permissions)')
 
     # The configuration file
-    namespace = {
-        'modules': modules,
-        'listen_port': getattr(options, 'port') or '8080',
-        'smtp_host': getattr(options, 'smtp_host') or 'localhost',
-        'smtp_from': email,
-    }
-    config = template.format(**namespace)
+    config = template.format(
+        modules=" ".join(modules),
+        listen_port=getattr(options, 'port') or '8080',
+        smtp_host=getattr(options, 'smtp_host') or 'localhost',
+        smtp_from=email,
+        log_email=getattr(options, 'log_email'))
     open('%s/config.conf' % target, 'w').write(config)
 
     # Create the folder structure
-    start_subprocess('%s/database' % target)
     database = make_database(target)
     mkdir('%s/log' % target)
     mkdir('%s/spool' % target)
 
     # Create a fake context
-    context = get_fake_context()
-    context.database = database
+    context = get_fake_context(database)
+    context.set_mtime = True
 
     # Make the root
-    folder = database.get_handler('.')
-    root = root_class._make_resource(root_class, folder, email, password)
+    metadata = Metadata(cls=root_class)
+    database.set_handler('.metadata', metadata)
+    root = root_class(metadata)
+    root.init_resource(email, password)
+    # Set mtime
+    root.set_property('mtime', context.timestamp)
     context.root = root
     # Save changes
+    context.git_message = 'Initial commit'
     database.save_changes()
-    # Index
+    # Index the root
     catalog = database.catalog
-    for resource in root.traverse_resources():
-        if isinstance(resource, DBResource):
-            catalog.index_document(resource)
     catalog.save_changes()
 
     # Bravo!
@@ -182,8 +204,12 @@ if __name__ == '__main__':
         help='use the given SMTP_HOST to send emails')
     parser.add_option('-w', '--password',
         help='use the given PASSWORD for the admin user')
+    parser.add_option('-m', '--modules', default='',
+        help='add the given MODULES to load at start')
     parser.add_option('--profile',
         help="print profile information to the given file")
+    parser.add_option('--log-email', default='',
+                      help='define the email address we will send errors')
 
     options, args = parser.parse_args()
     if len(args) != 1:
