@@ -23,32 +23,34 @@ from sys import exit
 
 # Import from itools
 from itools import __version__
-from itools.core import become_daemon, start_subprocess
+from itools.core import become_daemon
+from itools.database import check_database
+from itools.loop import Loop, cron
 
 # Import from ikaaro
-from ikaaro.database import check_database
 from ikaaro.update import is_instance_up_to_date
-from ikaaro.server import Server, get_pid, get_fake_context
+from ikaaro.server import Server, CMSContext, get_pid
 
 
 def start(options, target):
     # Check the server is not running
-    pid = get_pid(target)
+    pid = get_pid('%s/pid' % target)
     if pid is not None:
         print '[%s] The Web Server is already running.' % target
         return 1
+    # XXX Obsolete code, remove by 0.70
+    sub_pid = get_pid('%s/pid-subprocess' % target)
+    if sub_pid is not None:
+        print ('[%s] The Web Server subprocess is running, please use '
+               'icms-stop.py to stop it.') % target
+        return 1
 
     # Check for database consistency
     if options.quick is False and check_database(target) is False:
         return 1
 
-    # Set-up the server
-    server = Server(target, read_only=options.read_only)
-
     # Check instance is up to date
-    context = get_fake_context()
-    server.init_context(context)
-    if not is_instance_up_to_date(server.root):
+    if not is_instance_up_to_date(target):
         print 'The instance is not up-to-date, please type:'
         print
         print '    $ icms-update.py %s' % target
@@ -59,9 +61,38 @@ def start(options, target):
     if options.detach:
         become_daemon()
 
-    # Start
-    start_subprocess('%s/database' % target)
-    server.start()
+    # Set-up the server
+    server = Server(target, read_only=options.read_only,
+                    profile_space=options.profile_space)
+
+    # Update Git tree-cache, to speed things up
+    server.database.worktree.update_tree_cache()
+
+    # Find out the IP to listen to
+    config = server.config
+    address = config.get_value('listen-address').strip()
+    if not address:
+        raise ValueError, 'listen-address is missing from config.conf'
+    if address == '*':
+        address = None
+
+    # Find out the port to listen
+    port = config.get_value('listen-port')
+    if port is None:
+        raise ValueError, 'listen-port is missing from config.conf'
+
+    server.listen(address, port)
+    server.set_context('/', CMSContext)
+    interval = config.get_value('cron-interval')
+    if interval:
+        cron(server.cron_manager, 1)
+
+    # Run
+    profile = options.profile_time
+    profile = ('%s/log/profile' % target) if profile else None
+    loop = Loop(pid_file='%s/pid' % target, profile=profile)
+    loop.run()
+
     # Ok
     return 0
 
@@ -83,6 +114,12 @@ if __name__ == '__main__':
     parser.add_option(
         '--quick', action="store_true", default=False,
         help="Do not check the database consistency.")
+    parser.add_option(
+        '--profile-time', action="store_true", default=False,
+        help="Write profile information tot the 'log/profile' file.")
+    parser.add_option(
+        '--profile-space', action="store_true", default=False,
+        help="Enable remote monitoring by guppy, http://guppy-pe.sf.net/")
 
     # Parse arguments
     options, args = parser.parse_args()
