@@ -19,24 +19,32 @@
 
 # Import from the Standard Library
 from optparse import OptionParser
+from os import kill
+from signal import SIGINT, SIGTERM
 
 # Import from itools
 import itools
 
 # Import from ikaaro
-from ikaaro.server import Server
-from ikaaro.utils import kill
+from ikaaro.server import get_pid
 
 
 
 def stop(parser, options, target):
     # Stop the Web Server
-    server = Server(target, read_only=True)
-    pid = server.get_pid()
+    pid = get_pid('%s/pid' % target)
     if pid is None:
         print '[%s] Web Server not running.' % target
+
+        # XXX Obsolete code, remove by 0.70
+        # Eventually stop the subprocess
+        sub_pid = get_pid('%s/pid-subprocess' % target)
+        if sub_pid is not None:
+            kill(sub_pid, SIGTERM)
+            print '[%s] Web Server subprocess is running, i kill it' % target
     else:
-        kill(pid, force=options.force)
+        signal = SIGTERM if options.force else SIGINT
+        kill(pid, signal)
         if options.force:
             print '[%s] Web Server shutting down...' % target
         else:
