@@ -18,26 +18,26 @@
 
 # Import from the Standard Library
 from optparse import OptionParser
+from os import remove
+from os.path import join
+import sys
+from time import time
+from traceback import format_exc
 
 # Import from itools
 import itools
-from itools.core import start_subprocess
+from itools.core import vmsize
+from itools.database import check_database, get_register_fields
+from itools.fs import lfs
+from itools.database import make_catalog, Resource
+
+# Import from ikaaro
+from ikaaro.server import Server, ask_confirmation
+from ikaaro.server import get_fake_context
 
 
 
 def update_catalog(parser, options, target):
-    # Imports
-    import sys
-    from time import time
-    from itools.core import vmsize
-    from itools.i18n.accept import AcceptLanguage
-    from itools.fs import lfs
-    from itools.xapian import make_catalog, CatalogAware
-    from ikaaro.database import check_database
-    from ikaaro.server import Server, ask_confirmation
-    from ikaaro.server import get_fake_context
-    from ikaaro.registry import get_register_fields
-
     # Check the server is not started, or started in read-only mode
     server = Server(target, read_only=True, cache_size=options.cache_size)
     if server.is_running_in_rw_mode():
@@ -63,42 +63,67 @@ def update_catalog(parser, options, target):
     root = server.root
 
     # Build a fake context
-    context = get_fake_context()
-    context.accept_language = AcceptLanguage()
-    server.init_context(context)
+    context = get_fake_context(server.database)
+    context.server = server
 
     # Update
     t0, v0 = time(), vmsize()
     doc_n = 0
+    error_detected = False
+    if options.test:
+        log = open('%s/log/update-catalog' % target, 'w').write
     for obj in root.traverse_resources():
-        if not isinstance(obj, CatalogAware):
+        if not isinstance(obj, Resource):
             continue
         if not options.quiet:
-            print doc_n, obj.get_abspath()
+            print doc_n, obj.abspath
         doc_n += 1
         context.resource = obj
-        catalog.index_document(obj)
+
+        # Index the document
+        try:
+            catalog.index_document(obj)
+        except Exception:
+            if options.test:
+                error_detected = True
+                log('*** Error detected ***\n')
+                log('Abspath of the resource: %r\n\n' % str(obj.abspath))
+                log(format_exc())
+                log('\n')
+            else:
+                raise
+
         # Free Memory
         del obj
         server.database.make_room()
-    # Update / Report
-    t1, v1 = time(), vmsize()
-    v = (v1 - v0)/1024
-    print '[Update] Time: %.02f seconds. Memory: %s Kb' % (t1 - t0, v)
 
-    # Commit
-    print '[Commit]',
-    sys.stdout.flush()
-    catalog.save_changes()
-    # Commit / Replace
-    old_catalog_path = '%s/catalog' % target
-    if lfs.exists(old_catalog_path):
-        lfs.remove(old_catalog_path)
-    lfs.move(catalog_path, old_catalog_path)
-    # Commit / Report
-    t2, v2 = time(), vmsize()
-    v = (v2 - v1)/1024
-    print 'Time: %.02f seconds. Memory: %s Kb' % (t2 - t1, v)
+    if not error_detected:
+        if options.test:
+            # Delete the empty log file
+            remove('%s/log/update-catalog' % target)
+
+        # Update / Report
+        t1, v1 = time(), vmsize()
+        v = (v1 - v0)/1024
+        print '[Update] Time: %.02f seconds. Memory: %s Kb' % (t1 - t0, v)
+
+        # Commit
+        print '[Commit]',
+        sys.stdout.flush()
+        catalog.save_changes()
+        # Commit / Replace
+        old_catalog_path = '%s/catalog' % target
+        if lfs.exists(old_catalog_path):
+            lfs.remove(old_catalog_path)
+        lfs.move(catalog_path, old_catalog_path)
+        # Commit / Report
+        t2, v2 = time(), vmsize()
+        v = (v2 - v1)/1024
+        print 'Time: %.02f seconds. Memory: %s Kb' % (t2 - t1, v)
+    else:
+        print '[Update] Error(s) detected, the new catalog was NOT saved'
+        print ('[Update] You can find more infos in %r' %
+               join(target, 'log/update-catalog'))
 
 
 
@@ -121,7 +146,9 @@ if __name__ == '__main__':
         help="be quiet")
     parser.add_option(
         '--quick', action="store_true", default=False,
-        help="Do not check the database consistency.")
+        help="do not check the database consistency.")
+    parser.add_option('-t', '--test', action='store_true', default=False,
+        help="a test mode, don't stop the indexation when an error occurs")
 
     options, args = parser.parse_args()
     if len(args) != 1:
@@ -130,7 +157,6 @@ if __name__ == '__main__':
     target = args[0]
 
     # Action!
-    start_subprocess('%s/database' % target)
     if options.profile is not None:
         from cProfile import runctx
         runctx("update_catalog(parser, options, target)", globals(), locals(),
