@@ -19,47 +19,31 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
+from itools.core import proto_lazy_property
 from itools.gettext import get_domain
 from itools.i18n import get_language_name
-from itools.stl import STLTemplate
 from itools.uri import decode_query
 
 # Import from ikaaro
-from utils import reduce_string
-
-
-
-class SkinTemplate(STLTemplate):
-    template = None
-
-    def __init__(self, context):
-        self.context = context
-
-
-    def get_template(self):
-        template = self.template
-        if template is None:
-            msg = "%s is missing the 'template' variable"
-            raise NotImplementedError, msg % repr(self.__class__)
-        return self.context.root.get_resource(template)
+from utils import CMSTemplate, reduce_string
 
 
 
 ###########################################################################
 # Global language selector
 ###########################################################################
-class LanguagesTemplate(SkinTemplate):
+class LanguagesTemplate(CMSTemplate):
 
     template = '/ui/aruni/languages.xml'
 
 
-    def get_namespace(self):
+    @proto_lazy_property
+    def languages(self):
         context = self.context
         # Website languages
-        site_root = context.site_root
-        ws_languages = site_root.get_property('website_languages')
+        ws_languages = context.root.get_value('website_languages')
         if len(ws_languages) == 1:
-            return {'languages': []}
+            return []
 
         # Select language
         accept = context.accept_language
@@ -79,66 +63,82 @@ class LanguagesTemplate(SkinTemplate):
                 'selected': selected,
                 'class': css_class})
 
-        return {'languages': languages}
+        return languages
 
 
 ###########################################################################
 # Resource location & menu
 ###########################################################################
-class LocationTemplate(SkinTemplate):
+class LocationTemplate(CMSTemplate):
 
     template = '/ui/aruni/location.xml'
 
+    keep_view_and_query = False
 
-    def get_breadcrumb(self, context):
+    def get_url(self, path):
+        if not self.keep_view_and_query:
+            return path
+        uri = str(self.context.uri)
+        view_and_query = ''
+        if '/;' in uri:
+            view_and_query = ';' + uri.split('/;')[1]
+        return path + view_and_query
+
+
+    @proto_lazy_property
+    def breadcrumb(self):
         """Return a list of dicts [{name, url}...]
         """
-        site_root = context.site_root
+        context = self.context
+        root = context.root
 
         # Initialize the breadcrumb with the root resource
         path = '/'
-        title = site_root.get_title()
+        title = root.get_title()
+        if not title:
+            title = root.class_title.message.encode('utf_8')
         breadcrumb = [{
-            'url': path,
+            'url': self.get_url(path),
             'name': title,
-            'short_name': reduce_string(title, 15, 30),
-            }]
+            'short_name': reduce_string(title, 15, 30)}]
 
         # Complete the breadcrumb
-        resource = site_root
+        resource = root
         for name in context.uri.path:
             path = path + ('%s/' % name)
             resource = resource.get_resource(name, soft=True)
             if resource is None:
                 break
+            # Display resource title only if allowed
+            if not root.is_allowed_to_view(context.user, resource):
+                title = name
+            else:
+                title = resource.get_title()
             # Append
-            title = resource.get_title()
             breadcrumb.append({
-                'url': path,
+                'url': self.get_url(path),
                 'name': title,
-                'short_name': reduce_string(title, 15, 30),
-            })
+                'short_name': reduce_string(title, 15, 30)})
 
         return breadcrumb
 
 
-    def get_tabs(self, context):
+    @proto_lazy_property
+    def tabs(self):
         """Return tabs and subtabs as a dict {tabs, subtabs} of list of dicts
         [{name, label, active, style}...].
         """
-        # Get request, path, etc...
+        # Get resource & access control
+        context = self.context
+        if context.user is None:
+            return []
+
         here = context.resource
         here_link = context.get_link(here)
 
-        # Get access control
-        user = context.user
-        ac = here.get_access_control()
-
         # Tabs
         tabs = []
         for link, view in here.get_views():
-            active = False
-
             # From method?param1=value1&param2=value2&...
             # we separate method and arguments, then we get a dict with
             # the arguments and the subview active state
@@ -149,21 +149,19 @@ class LocationTemplate(SkinTemplate):
                 name, args = link, {}
 
             # Active
-            if context.view == here.get_view(name, args):
-                active = True
+            unbound_view = context.view.__bases__[0]
+            active = (unbound_view is here.get_view(name, args).__bases__[0])
 
             # Add the menu
             tabs.append({
                 'name': '%s/;%s' % (here_link, link),
                 'label': view.get_title(context),
                 'active': active,
-                'class': active and 'active' or None})
+                'class': 'active' if active else None})
 
         return tabs
 
 
-    def get_namespace(self):
-        context = self.context
-        return {
-            'breadcrumb': self.get_breadcrumb(context),
-            'tabs': self.get_tabs(context)}
+    @proto_lazy_property
+    def location(self):
+        return bool(self.breadcrumb) or bool(self.tabs)
