@@ -2,6 +2,7 @@
 # Copyright (C) 2006-2007 Hervé Cauwelier <herve@itaapy.com>
 # Copyright (C) 2006-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
 # Copyright (C) 2008 Nicolas Deram <nicolas@itaapy.com>
+# Copyright (C) 2011 Armel FORTUN <armel@tchack.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,17 +21,16 @@
 from operator import itemgetter
 
 # Import from itools
-from itools.core import merge_dicts
-from itools.datatypes import Integer, String
+from itools.datatypes import Integer, String, Unicode
 from itools.gettext import MSG
-from itools.web import STLForm, STLView, INFO
+from itools.handlers import TextFile
+from itools.web import STLView, INFO
 
 # Import from ikaaro
-from buttons import RemoveButton
+from autoform import AutoForm, EditAreaWidget, get_default_widget
+from buttons import Button, Remove_BrowseButton
+from fields import File_Field
 from file_views import File_Edit
-from forms import AutoForm, get_default_widget, MultilineWidget
-from forms import timestamp_widget, description_widget, file_widget
-from forms import subject_widget, title_widget
 import messages
 from views import BrowseForm
 
@@ -39,27 +39,37 @@ class Text_Edit(File_Edit):
 
     title = MSG(u'Edit')
     icon = 'edit.png'
-    schema = merge_dicts(File_Edit.schema, data=String)
-    widgets = [
-        timestamp_widget,
-        title_widget,
-        MultilineWidget('data', title=MSG(u"Content"), rows=19, cols=69),
-        file_widget,
-        description_widget,
-        subject_widget]
+
+    fields = ['title', 'data', 'file', 'description', 'subject', 'share']
 
 
     def get_value(self, resource, context, name, datatype):
         if name == 'data':
-            return resource.handler.to_str()
-        return File_Edit.get_value(self, resource, context, name, datatype)
+            data = resource.get_value('data')
+            if data is None:
+                return ''
+            return data.to_str()
+        proxy = super(Text_Edit, self)
+        return proxy.get_value(resource, context, name, datatype)
 
 
-    def action(self, resource, context, form):
-        File_Edit.action(self, resource, context, form)
-        if form['file'] is None:
-            data = form['data']
-            resource.handler.load_state_from_string(data)
+    def set_value(self, resource, context, name, form):
+        if name == 'data':
+            if form.get('file'):
+                return False
+            old_data = resource.get_value('data')
+            old_value = old_data.to_str()
+            value = form['data']
+            # Remove ^M character
+            value = '\n'.join(value.splitlines())
+            if old_value == value:
+                return False
+            # Save new text
+            old_data.load_state_from_string(value)
+            context.database.change_resource(resource)
+            return False
+
+        return super(Text_Edit, self).set_value(resource, context, name, form)
 
 
 
@@ -72,19 +82,32 @@ class Text_View(STLView):
 
 
     def get_namespace(self, resource, context):
-        return {'data': resource.handler.to_str()}
+        data = resource.get_value('data')
+        return {'data': data.to_str() if data else ''}
 
 
 
-class PO_Edit(STLForm):
+class CSS_Edit(Text_Edit):
+    """Code editor view using the EditArea instead of a basic textarea or
+    the WYSIWYG tinyMCE (that cannot directly edit source, as far as I known
+    [Armel]). Mix CSS_Edit and HTMLEditView"""
+
+    title = "Edit CSS file"
+
+    data = File_Field(required=True, class_handler=TextFile,
+        widget=EditAreaWidget)
+
+
+
+class PO_Edit(STLView):
 
     access = 'is_allowed_to_edit'
     title = MSG(u'Edit')
     template = '/ui/PO_edit.xml'
     schema = {
-        'msgid': String(mandatory=True),
-        'msgstr': String(mandatory=True),
-    }
+        'msgctxt': Unicode,
+        'msgid': Unicode(mandatory=True),
+        'msgstr': Unicode(mandatory=True)}
 
 
     def get_namespace(self, resource, context):
@@ -105,9 +128,11 @@ class PO_Edit(STLForm):
         # Msgid and msgstr
         if units:
             unit = units[index-1]
-            msgid = ''.join(unit.source)
-            msgstr = ''.join(unit.target)
+            msgctxt = u'' if unit.context is None else ''.join(unit.context)
+            msgid = u''.join(unit.source)
+            msgstr = u''.join(unit.target)
         else:
+            msgctxt = None
             msgid = None
             msgstr = None
 
@@ -120,14 +145,16 @@ class PO_Edit(STLForm):
             'messages_last': uri.replace(messages_index=str(total)),
             'messages_previous': uri.replace(messages_index=str(previous)),
             'messages_next': uri.replace(messages_index=str(next)),
+            'msgctxt': msgctxt,
             'msgid': msgid,
             'msgstr': msgstr}
 
 
     def action(self, resource, context, form):
+        msgctxt = None if not form['msgctxt'] else form['msgctxt']
         msgid = form['msgid'].replace('\r', '')
         msgstr = form['msgstr'].replace('\r', '')
-        resource.handler.set_message(msgid, msgstr)
+        resource.handler.set_msgstr(msgid, msgstr, msgctxt)
         # Events, change
         context.database.change_resource(resource)
 
@@ -145,6 +172,9 @@ class CSV_View(BrowseForm):
         'ids': Integer(mandatory=True, multiple=True),
     }
 
+    search_schema = {}
+    search_widgets = []
+
 
     def get_items(self, resource, context):
         return list(resource.handler.get_rows())
@@ -197,7 +227,7 @@ class CSV_View(BrowseForm):
         return value
 
 
-    table_actions = [RemoveButton]
+    table_actions = [Remove_BrowseButton]
 
 
     def action_remove(self, resource, context, form):
@@ -227,8 +257,7 @@ class RowForm(AutoForm):
         schema = self.get_schema(resource, context)
         return [
             get_default_widget(schema[name])(name, title=title)
-            for name, title in resource.get_columns()
-        ]
+            for name, title in resource.get_columns() ]
 
 
 
@@ -236,7 +265,7 @@ class CSV_AddRow(RowForm):
 
     title = MSG(u'Add Row')
     icon = 'new.png'
-    submit_value = MSG(u'Add')
+    actions = [Button(access=True, css='button-ok', title=MSG(u'Add'))]
 
 
     def action(self, resource, context, form):
