@@ -15,6 +15,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-def is_instance_up_to_date(root):
-    # 0.60 Check the Git archive has been initalized
-    return root.metadata.database.fs.exists('.git')
+def is_instance_up_to_date(target):
+    # 0.62 Check for the new metadata
+    metadata = open('%s/database/.metadata' % target).read()
+    return not metadata.startswith('<?xml')
